#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <string.h>
#include <dwarf.h>
#include <stdio.h>

#define BACKEND arm_
#include "libebl_CPU.h"

ssize_t
arm_register_info (Ebl *ebl __attribute__((unused)),
		   int regno, char *name, size_t namelen,
		   const char **prefix, const char **setname,
		   int *bits, int *type)
{
  if (name == NULL)
    return 288;

  if (regno < 0 || regno > 287 || namelen < 9)
    return -1;

  *prefix = NULL;

  *type = DW_ATE_unsigned;
  if (regno < 16)
    {
      *setname = "integer";
      *bits = 32;
      if (regno < 13)
        *type = DW_ATE_signed;
      else
        *type = DW_ATE_address;
    }
  else if (regno < 24)
    {
      *setname = "FPA";
      *bits = 64;
      *type = DW_ATE_float;
    }
  else if (regno >= 64 && regno < 96)
    {
      *setname = "VFP";
      *bits = 64;
      *type = DW_ATE_float;
    }
  else if (regno >= 96 && regno < 104)
    {
      *setname = "FPA";
      *bits = 64;
      *type = DW_ATE_float;
    }
  else if (regno >= 104 && regno < 128)
    {
      *setname = "MMX";
      *bits = 64;
    }
  else if (regno >= 128 && regno < 134)
    {
      *setname = "control";
      *bits = 32;
    }
  else if (regno >= 144 && regno < 166)
    {
      *setname = "integer";
      *bits = 32;
    }
  else if (regno >= 192 && regno < 200)
    {
      *setname = "MMX";
      *bits = 64;
    }
  else if (regno >= 256 && regno < 288)
    {
      *setname = "NEON";
      *bits = 64;
    }

  switch (regno)
    {
    case 0 ... 9:
      name[0] = 'r';
      name[1] = (regno - 0) + '0';
      namelen = 2;
      break;

    case 10 ... 10 + 5:
      name[0] = 'r';
      name[1] = '1';
      name[2] = (regno - 10) + '0';
      namelen = 3;
      break;

    case 16 + 0 ... 16 + 7:
      name[0] = 'f';
      name[1] = (regno - 16) + '0';
      namelen = 2;
      break;

    case 64 + 0 ... 64 + 9:
      name[0] = 's';
      name[1] = (regno - 64) + '0';
      namelen = 2;
      break;

    case 64 + 10 ... 64 + 31:
      name[0] = 's';
      name[1] = (regno - 64) / 10 + '0';
      name[2] = (regno - 64) % 10 + '0';
      namelen = 3;
      break;

    case 96 + 0 ... 96 + 7:
      name[0] = 'f';
      name[1] = (regno - 96) + '0';
      namelen = 2;
      break;

    case 104 ... 104 + 7:
      memcpy(name, "wcgr", 4);
      name[4] = (regno - 104) + '0';
      namelen = 5;
      break;

    case 112 + 0 ... 112 + 9:
      name[0] = 'w';
      name[1] = 'r';
      name[2] = (regno - 112) + '0';
      namelen = 3;
      break;

    case 112 + 10 ... 112 + 15:
      name[0] = 'w';
      name[1] = 'r';
      name[2] = '1';
      name[3] = (regno - 112 - 10) + '0';
      namelen = 4;
      break;

    case 128:
      return stpcpy (name, "spsr") + 1 - name;
    case 129:
      return stpcpy (name, "spsr_fiq") + 1 - name;
    case 130:
      return stpcpy (name, "spsr_irq") + 1 - name;
    case 131:
      return stpcpy (name, "spsr_abt") + 1 - name;
    case 132:
      return stpcpy (name, "spsr_und") + 1 - name;
    case 133:
      return stpcpy (name, "spsr_svc") + 1 - name;

    case 144 ... 150:
      return sprintf(name, "r%d_usr", (regno - 144) + 8) + 1;
    case 151 ... 157:
      return sprintf(name, "r%d_fiq", (regno - 151) + 8) + 1;
    case 158 ... 159:
      return sprintf(name, "r%d_irq", (regno - 158) + 13) + 1;
    case 160 ... 161:
      return sprintf(name, "r%d_abt", (regno - 160) + 13) + 1;
    case 162 ... 163:
      return sprintf(name, "r%d_und", (regno - 162) + 13) + 1;
    case 164 ... 165:
      return sprintf(name, "r%d_svc", (regno - 164) + 13) + 1;

    case 192 ... 199:
      name[0] = 'w';
      name[1] = 'c';
      name[2] = (regno - 192) + '0';
      namelen = 3;
      break;

    case 256 + 0 ... 256 + 9:
      name[0] = 'd';
      name[1] = (regno - 256) + '0';
      namelen = 2;
      break;

    case 256 + 10 ... 256 + 31:
      name[0] = 'd';
      name[1] = (regno - 256) / 10 + '0';
      name[2] = (regno - 256) % 10 + '0';
      namelen = 3;
      break;

    default:
      *setname = NULL;
      return 0;
    }

  name[namelen++] = '\0';
  return namelen;
}
